@@ -8,7 +8,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)cxx_table.cpp	10.17 (Sleepycat) 10/29/98";
+static const char sccsid[] = "@(#)cxx_table.cpp	10.18 (Sleepycat) 11/22/98";
 #endif /* not lint */
 
 #include "db_cxx.h"
@@ -52,7 +52,7 @@ int Db::close(u_int32_t flags)
     return 0;
 }
 
-int Db::cursor(DbTxn *txnid, Dbc **cursorp)
+int Db::cursor(DbTxn *txnid, Dbc **cursorp, u_int32_t flags)
 {
     DB *db = unwrap(this);
     int err;
@@ -62,7 +62,7 @@ int Db::cursor(DbTxn *txnid, Dbc **cursorp)
         return EINVAL;
     }
     DBC *dbc = 0;
-    if ((err = db->cursor(db, unwrap(txnid), &dbc)) != 0) {
+    if ((err = db->cursor(db, unwrap(txnid), &dbc, flags)) != 0) {
         DB_ERROR("Db::cursor", err);
         return err;
     }
@@ -167,6 +167,23 @@ int Db::open(const char *fname, DBTYPE type, u_int32_t flags,
     return 0;
 }
 
+// static method
+int Db::xa_open(const char *fname, DBTYPE type, u_int32_t flags,
+             int mode, DbInfo *info, Db **table_returned)
+{
+    *table_returned = 0;
+    DB *newtable;
+    int err;
+    if ((err = db_xa_open(fname, type, flags, mode,
+                       info, &newtable)) != 0) {
+        DB_ERROR("Db::open", err);
+        return err;
+    }
+    *table_returned = new Db();
+    (*table_returned)->imp_ = wrap(newtable);
+    return 0;
+}
+
 int Db::put(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
 {
     DB *db = unwrap(this);
