@@ -39,6 +39,7 @@ static int  __db_xa_prepare __P((XID *, int, long));
 static int  __db_xa_recover __P((XID *, long, int, long));
 static int  __db_xa_rollback __P((XID *, int, long));
 static int  __db_xa_start __P((XID *, int, long));
+static void __xa_txn_end __P((DB_ENV *));
 static void __xa_txn_init __P((DB_ENV *, TXN_DETAIL *, size_t));
 
 /*
@@ -184,7 +185,7 @@ __db_xa_start(xid, rmid, flags)
 	if (LF_ISSET(~OK_FLAGS))
 		return (XAER_INVAL);
 
-	if (LF_ISSET(TMJOIN) || LF_ISSET(TMRESUME))
+	if (LF_ISSET(TMJOIN) && LF_ISSET(TMRESUME))
 		return (XAER_INVAL);
 
 	if (LF_ISSET(TMASYNC))
@@ -330,7 +331,8 @@ __db_xa_prepare(xid, rmid, flags)
 
 	td->xa_status = TXN_XA_PREPARED;
 
-	/* No fatal value that would require an XAER_RMFAIL); */
+	/* No fatal value that would require an XAER_RMFAIL. */
+	__xa_txn_end(env);
 	return (XA_OK);
 }
 
@@ -376,6 +378,7 @@ __db_xa_commit(xid, rmid, flags)
 	if (LF_SET(TMONEPHASE) &&
 	    td->xa_status != TXN_XA_ENDED && td->xa_status != TXN_XA_SUSPENDED)
 		return (XAER_PROTO);
+
 	if (!LF_SET(TMONEPHASE) && td->xa_status != TXN_XA_PREPARED)
 		return (XAER_PROTO);
 
@@ -385,7 +388,8 @@ __db_xa_commit(xid, rmid, flags)
 	if (txn_commit(env->xa_txn) != 0)
 		return (XAER_RMERR);
 
-	/* No fatal value that would require an XAER_RMFAIL); */
+	/* No fatal value that would require an XAER_RMFAIL. */
+	__xa_txn_end(env);
 	return (XA_OK);
 }
 
@@ -553,6 +557,7 @@ __db_xa_rollback(xid, rmid, flags)
 	if (LF_SET(TMONEPHASE) &&
 	    td->xa_status != TXN_XA_ENDED && td->xa_status != TXN_XA_SUSPENDED)
 		return (XAER_PROTO);
+
 	if (!LF_SET(TMONEPHASE) && td->xa_status != TXN_XA_PREPARED)
 		return (XAER_PROTO);
 
@@ -561,7 +566,8 @@ __db_xa_rollback(xid, rmid, flags)
 	if (txn_abort(env->xa_txn) != 0)
 		return (XAER_RMERR);
 
-	/* No fatal value that would require an XAER_RMFAIL); */
+	/* No fatal value that would require an XAER_RMFAIL. */
+	__xa_txn_end(env);
 	return (XA_OK);
 }
 
@@ -597,7 +603,7 @@ __db_xa_forget(xid, rmid, flags)
 
 	__db_unmap_xid(env, xid, off);
 
-	/* No fatal value that would require an XAER_RMFAIL); */
+	/* No fatal value that would require an XAER_RMFAIL. */
 	return (XA_OK);
 }
 
@@ -640,3 +646,18 @@ __xa_txn_init(env, td, off)
 	txn->off = off;
 	txn->flags = 0;
 }
+
+/*
+ * __xa_txn_end --
+ * 	Invalidate a transaction structure that was generated by xa_txn_init.
+ */
+static void
+__xa_txn_end(env)
+	DB_ENV *env;
+{
+	DB_TXN *txn;
+
+	txn = env->xa_txn;
+	if (txn != NULL)
+		txn->txnid = TXN_INVALID;
+}
